/* ESP8266_uMQTT_DHT2
 *  
 * MQTT kliens program, amely egy DHT22 szenzor adatait publikálja 
 * a helyi hálózaton a cspista/home/temp és cspista/home/humidity
 * témakörökbe.
 * A kliens ugyanakkor feliratkozik a cspista/home/led témakörre és
 * a beérkező üzenetek első karakterével (0 vagy 1) a beépített LED-et
 * be- vagy kikapcsoljuk.
 * 
 * Az MQTT brókerhez a secrets.h-ba uMQTT_USER és uMQTT_PASS névvel
 * definiált autentikációs adatokkal jeletkezünk be.
 * 
 * Tested with Arduino IDE 1.8.13 and ESP8266 Arduino core 2.7.4 
 */
#include <ESP8266WiFi.h>
#include <PubSubClient.h>
#include "DHT.h"
#include "secrets.h"
const char* mqtt_server = "192.168.1.11";
#define DHTPIN          14     // GPIO14 (D5) pin for DHT sensor

DHT dht(DHTPIN, DHT22);
WiFiClient espClient;
PubSubClient client(espClient);

unsigned long lastMsg = 0;
#define MSG_BUFFER_SIZE	(50)
char msg[MSG_BUFFER_SIZE];
int value = 0;
int nClient = 0;

void setup_wifi() {
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(WIFI_SSID);
  WiFi.mode(WIFI_STA);
  WiFi.begin(WIFI_SSID, WIFI_PASS);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("WiFi connected");
  Serial.print("IP address: ");
  Serial.println(WiFi.localIP());
}

void callback(char* topic, byte* payload, unsigned int length) {
  Serial.print("Message arrived [");
  Serial.print(topic);
  Serial.print("] : ");
  for (int i = 0; i < length; i++) {
    Serial.print((char)payload[i]);
  }
  Serial.println();

  // Switch on the LED if an 1 was received as first character
  if ((char)payload[0] == '1') {
    digitalWrite(BUILTIN_LED, LOW);   // Turn the LED on (Note that LOW is the voltage level
    // but actually the LED is on; this is because
    // it is active low on the ESP-01)
  } else {
    digitalWrite(BUILTIN_LED, HIGH);  // Turn the LED off by making the voltage HIGH
  }
}

void reconnect() {
  // Loop until we're reconnected
  while (!client.connected()) {
    Serial.print("Attempting MQTT connection...");
    // Create a random client ID
    char clientId[] = "hobbi@cspista.hu";
    // Attempt to connect
    if (client.connect(clientId, uMQTT_USER, uMQTT_PASS)) {
      Serial.println("connected");
      // Once connected, publish an announcement...
      client.publish("cspista/home", "DHT22 connected");
      // ... and resubscribe
      client.subscribe("cspista/home/led");
    } else {
      Serial.print("failed, rc=");
      Serial.print(client.state());
      Serial.println(" try again in 5 seconds");
      // Wait 5 seconds before retrying
      delay(5000);
    }
  }
}

void setup() {
  pinMode(BUILTIN_LED, OUTPUT);     // Initialize the BUILTIN_LED pin as an output
  Serial.begin(115200);
  dht.begin();                      // Initialize the DHT22 sensor
  setup_wifi();
  client.setServer(mqtt_server, 1883);
  client.setCallback(callback);
}

void loop() {
  if (!client.connected()) {
    reconnect();
  }
  client.loop();
  unsigned long now = millis();
  if (now - lastMsg > 5000) {
    lastMsg = now;
    float t = dht.readTemperature();
    if (!isnan(t)) {
      sprintf(msg, "%5.1f", t);
      client.publish("cspista/home/temp", msg);
      Serial.print("Publish [cspista/home/temp]:");
      Serial.println(msg);
    }
    float h = dht.readHumidity();
    if (!isnan(h)) {
      sprintf(msg, "%3.0f", h);
      client.publish("cspista/home/humidity", msg);
      Serial.print("Publish [cspista/home/humidity]:");
      Serial.println(msg);
    }
  }
}
